package com.flyfish.framework.auditor;

import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.generation.CodeRules;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Mono;

/**
 * 异步公共审查
 *
 * @author wangyu
 */
@Component
public class ReactiveCommonBeanAuditor implements ReactiveBeanAuditor<Domain> {

    /**
     * 对实体进行审查，并补全相关字段
     *
     * @param data 原数据
     * @return 结果
     */
    @Override
    public Mono<Domain> audit(Domain data) {
        // 编码审查，自动生成
        if (StringUtils.isBlank(data.getId()) && StringUtils.isBlank(data.getCode())) {
            return CodeRules.retrieve(data)
                    .map(code -> {
                        data.setCode(code);
                        return data;
                    })
                    .thenReturn(data);
        }
        return Mono.just(data);
    }
}
