package com.flyfish.framework.domain.base;

import com.fasterxml.jackson.annotation.JsonIgnore;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.repository.core.EntityInformation;

import java.util.List;
import java.util.Optional;

/**
 * 查询模型
 *
 * @param <T> 泛型
 */
public interface Qo<T> {

    /**
     * 查询键
     */
    String KEY = "query";

    /**
     * 获取分页对象
     *
     * @return 结果
     */
    Pageable getPageable();

    /**
     * 设置分页对象
     *
     * @param pageable 分页对象
     */
    @JsonIgnore
    void setPageable(Pageable pageable);

    /**
     * 获取当前用户
     *
     * @return 结果
     */
    IUser getUser();

    /**
     * 当前用户
     */
    void setUser(IUser user);

    /**
     * 获取结果集，用于单例部署时快速封装
     *
     * @return 结果
     */
    List<T> getResult();

    /**
     * 设置结果集，用于单例部署时快速封装
     *
     * @param result 查询结果集
     */
    @JsonIgnore
    void setResult(List<T> result);

    /**
     * 获取jpa的example对象
     *
     * @return 结果
     */
    @JsonIgnore
    Example<T> getExample();

    /**
     * 获取spring data的对象Predicate
     *
     * @param entityInformation 实体信息，来自上下文
     * @return 结果
     */
    @JsonIgnore
    <C> Optional<C> getQuery(EntityInformation<T, String> entityInformation);

    /**
     * 获取查询的字段
     *
     * @return 结果
     */
    @JsonIgnore
    List<String> getFields();

    /**
     * 是否拉取关联查询
     *
     * @return 结果
     */
    @JsonIgnore
    boolean isFetchRef();

    /**
     * 让值全部包含在Pojo里
     *
     * @return 结果
     */
    Class<T> pojoType();

    /**
     * 排序字段，默认createTime
     *
     * @return 结果
     */
    Sort sorts();

    /**
     * 判断当前查询是否为空
     *
     * @return 结果
     */
    boolean isEmpty();
}
