package com.flyfish.framework.domain.base;

import com.flyfish.framework.context.ViewModelContext;
import reactor.core.publisher.Mono;

/**
 * 基础的视图模型
 *
 * @param <T> 泛型
 */
public interface Vo<T> {

    String getId();

    void setId(String id);

    /**
     * 赋值，从po转换的逻辑
     *
     * @param po 数据库实体
     * @return 结果
     */
    default Vo<T> from(T po) {
        return this;
    }

    /**
     * 异步的进行查询处理
     *
     * @param po 实体
     * @return 结果
     */
    default Mono<Vo<T>> retrieve(T po) {
        return Mono.just(from(po));
    }

    /**
     * 最终的wrap方法，用于全局拦截
     *
     * @param po 数据库实体
     * @return 结果
     */
    default Mono<Vo<T>> wrap(T po) {
        return retrieve(po).flatMap(ViewModelContext::intercept);
    }
}
