package com.flyfish.framework.domain.po;

import com.flyfish.framework.domain.base.AuditDomain;
import com.flyfish.framework.enums.NamedEnum;
import com.flyfish.framework.enums.RoleType;
import com.flyfish.framework.relational.mapping.Association;
import lombok.*;
import org.springframework.data.mongodb.core.mapping.DBRef;
import org.springframework.data.mongodb.core.mapping.Document;
import org.springframework.data.relational.core.mapping.Table;

import java.util.List;

/**
 * 角色（权限）
 *
 * @author wangyu
 */
@Document("roles")
@Table("t_role")
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class Role extends AuditDomain {

    /**
     * 描述
     */
    private String description;

    /**
     * 是否管理员
     */
    private Boolean admin;

    /**
     * 是否系统内置
     */
    private boolean system;

    /**
     * 所属平台
     */
    private RoleType type;

    /**
     * 角色拥有的权限
     */
    @DBRef
    @Association(relationTable = "rel_role_permission", field = "role_id", foreignField = "permission_id")
    private List<Permission> permissions;

    /**
     * 角色拥有的数据权限
     */
    private List<Authority> authorities;

    /**
     * 数据规则权限
     */
    @AllArgsConstructor
    @Getter
    public enum Authority implements NamedEnum {

        ADMIN("拥有完全控制权限"), VIEW("查看本部门"), EDIT("编辑本部门"),
        VIEW_CHILDREN("查看所有下级"), EDIT_CHILDREN("编辑所有下级");

        private final String name;
    }
}
