package com.flyfish.framework.mongodb.query.converter;

import com.flyfish.framework.query.spi.adaptor.CriteriaAdaptor;
import com.flyfish.framework.query.spi.converter.QueryConverter;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;

@RequiredArgsConstructor
public class MongoQueryConverter implements QueryConverter<Query> {

    private final CriteriaAdaptor<Criteria> adaptor;

    /**
     * 转换查询
     *
     * @param query 封装的查询
     * @return 本地查询
     */
    @Override
    public Query convert(com.flyfish.framework.query.Query query) {
        Criteria condition = Criteria.where("delete").ne(true);
        // 添加条件
        Criteria criteria = getCriteria(query);
        if (null != criteria) {
            condition = condition.andOperator(criteria);
        }
        // 构建查询
        Query result = Query.query(condition);
        if (null != query.getSort()) {
            result = result.with(query.getSort());
        }
        if (null != query.getPageable()) {
            result = result.with(query.getPageable());
        }
        if (CollectionUtils.isNotEmpty(query.getFields())) {
            result.fields().include(query.getFields().toArray(new String[]{}));
        }
        return result;
    }

    /**
     * 获取真实条件
     *
     * @param query 查询信息
     * @return 结果
     */
    private Criteria getCriteria(com.flyfish.framework.query.Query query) {
        // 当且仅当有example且正确生成条件时，才允许使用example构造
        if (null != query.getExample()) {
            return new Criteria().alike(query.getExample());
        }
        // 如果存在查询定义，直接使用查询定义
        if (null != query.getDefinition()) {
            return query.getDefinition().build(adaptor);
        }
        return null;
    }

}
