package com.flyfish.framework.r2dbc.metadata.reference.field;

import com.flyfish.framework.domain.base.Domain;
import org.springframework.beans.BeanUtils;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;

/**
 * 字段访问器
 *
 * @author wangyu
 */
public interface FieldAccessor {

    static FieldAccessor of(Class<?> type, Field field) {
        PropertyDescriptor descriptor = BeanUtils.getPropertyDescriptor(type, field.getName());
        return new DefaultFieldAccessor(descriptor);
    }

    /**
     * 设置字段值
     *
     * @param obj   具体对象
     * @param value 值
     * @param <T>   对象的泛型
     * @return 设置后的对象
     */
    <T extends Domain> T set(T obj, Object value);

    /**
     * 取得字段值
     *
     * @param obj 具体对象
     * @param <T> 对象的泛型
     * @param <R> 值的泛型
     * @return 具体字段值
     */
    <T extends Domain, R> R get(T obj);

    /**
     * setter是否可用
     *
     * @return 结果
     */
    boolean hasSetter();

    /**
     * getter是否可用
     *
     * @return 结果
     */
    boolean hasGetter();

    /**
     * 是否可用，即支持读写
     *
     * @return 结果
     */
    default boolean isAvailable() {
        return hasGetter() && hasSetter();
    }
}
