package com.flyfish.framework.user.controller;

import com.flyfish.framework.bean.Result;
import com.flyfish.framework.controller.reactive.ReactiveTreeController;
import com.flyfish.framework.user.service.DepartmentService;
import com.flyfish.framework.user.domain.AdminUserDetails;
import com.flyfish.framework.user.domain.DepartmentQo;
import com.flyfish.framework.domain.po.Department;
import com.flyfish.framework.enums.UserType;
import com.flyfish.framework.utils.DepartUtils;
import com.flyfish.framework.utils.UserUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.security.core.context.ReactiveSecurityContextHolder;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * 部分或者校区controller
 */
@RestController
@RequestMapping("/departments")
public class DepartmentController extends ReactiveTreeController<Department, DepartmentQo> {

    /**
     * 获取有权限的tree
     *
     * @return 结果
     */
    @GetMapping("authorized-tree")
    public Mono<Result<List<Department>>> getAuthorizedTree() {
        return ReactiveSecurityContextHolder.getContext()
                .map(UserUtils::extractUserDetails)
                .flatMapMany(detail -> {
                    AdminUserDetails details = (AdminUserDetails) detail;
                    if (details.getType() == UserType.SUPER_ADMIN) {
                        return reactiveService.getAll();
                    }
                    if (CollectionUtils.isNotEmpty(details.getDepartments())) {
                        DepartmentService departmentService = getService();
                        Set<String> departs = DepartUtils.mergeDeparts(details.getDepartments());
                        return departmentService.getSubCodes(departs)
                                .flatMapMany(codes -> {
                                    List<String> merged = new ArrayList<>();
                                    merged.addAll(codes);
                                    merged.addAll(departs);
                                    return departmentService.getByIds(merged);
                                });
                    }
                    return Flux.empty();
                })
                .collectList()
                .map(this::makeTree)
                .map(Result::accept);
    }
}
